import React from 'react';
import { Container, Row, Col } from 'reactstrap';
import { Form } from 'formik';
import { Link } from 'react-router-dom';
import ReportMap from '../map/ReportMap';
import { client, customEvents } from '../../functions/eventEmitter';
import { connect } from 'react-redux';
import reportAction from '../../actions/traffic-status';
import { Spin } from 'antd';
import { AiFillHome } from 'react-icons/ai';
import _ from 'lodash';
import {
  Select,
  MenuItem,
  FormControl,
  InputLabel,
  TextField,
  FormHelperText,
  Button,
  InputAdornment,
} from '@material-ui/core';
import SegmentPopover from '../shared/SegmentPopover';
import constants from '../../constants';

const CascaderReasonOptions = [
  { value: '', label: '-----', key: 0 },
  { value: 'TJ', label: 'Tắc đường', key: 1 },
  { value: 'FL', label: 'Ngập lụt', key: 2 },
  { value: 'OB', label: 'Vật cản', key: 3 },
  { value: 'AC', label: 'Tai nạn', key: 4 },
  { value: 'PO', label: 'Công an', key: 5 },
  { value: 'FR', label: 'Đường cấm', key: 6 },
];

const CascaderWeatherOptions = [
  { value: '', label: '-----', key: 0},
  { value: 'clear-sky', label: 'Trời đẹp', key: 1 },
  { value: 'scattered-clouds', label: 'Nhiều mây', key: 2 },
  { value: 'broken-clouds', label: 'Nhiều mây đen', key: 3 },
  { value: 'light-rain', label: 'Mưa nhỏ', key: 4 },
  { value: 'heavy-rain', label: 'Mưa lớn', key: 5 }, 
  { value: 'mist', label: 'Sương mù', key: 6}
]

class TrafficReportForm extends React.Component {
  constructor(props, context) {
    super(props, context);
    this.reportMap = null;
    this.mapContainerRef = React.createRef();
  }

  setReportState = (state) => {
    this.props.setReportState(state);
  };

  onInput = (key) => (event) => {
    const value = event.target.value;
    const { error } = this.props.trafficReport.report;
    delete error[key];
    if (key === 'velocity') {
      if (parseInt(value) > 40) {
        error.velocity = 'Vận tốc không được lớn hơn 40 km/h';
      }
      if (parseInt(value) < 0) {
        error.velocity = 'Vận tốc không được bé hơn 0';
      }
    }
    this.setReportState({
      ['report.' + key]: value,
      'report.error': { ...error },
    });
  };

  handleAfterSubmit = () => {
    client.emit(customEvents.reportSuccess);
    this.setState({
      start: {},
      end: {},
      addressData: [
        constants.addressDataStartDefault,
        constants.addressDataEndDefault,
      ],
      velocity: '',
      weather: '',
      reason: '',
      description: '',
    });
    this.props.cleanReportState('report');
    if (this.reportMap) {
      this.reportMap.onRemoveLayer();
    }
  };

  onSubmit = (e) => {
    e.preventDefault();
    const {
      markerData,
      velocity,
      weather,
      reason,
      description,
      error,
      multiReportData,
      isDrawable,
    } = this.props.trafficReport.report;

    let errors = { ...error };
    if (isDrawable) {
      const { reportMultiSegment } = this.props;
      const report = multiReportData.map((item) => {
        if (!item.velocity) {
          if (!velocity && typeof velocity !== 'number') {
            errors.velocity = 'Chưa nhập vận tốc';
          }
        }
        return {
          ...item,
          velocity: item.velocity || velocity,
          weather: weather,
          causes: reason,
          description: description,
        };
      });
      if (!_.isEmpty(errors)) {
        this.setReportState({ 'report.error': errors });
        return;
      } else {
        reportMultiSegment({ report }).then((success) => {
          if (success) {
            this.handleAfterSubmit();
          }
        });
        return;
      }
    }
    if (!velocity && typeof velocity !== 'number') {
      errors.velocity = 'Chưa nhập vận tốc';
    }
    const [start, end] = markerData;

    if (!start || _.isEmpty(start)) {
      errors.start = 'Chưa chọn vị trí bắt đầu';
    }
    if (!end || _.isEmpty(end)) {
      errors.end = 'Chưa chọn vị trí kết thúc';
    }
    if (!_.isEmpty(errors)) {
      this.setReportState({ 'report.error': errors });
    } else {
      let data = {
        currentLat: start.lat.toString(),
        currentLng: start.lng.toString(),
        nextLat: end.lat.toString(),
        nextLng: end.lng.toString(),
        velocity,
      };
      if (description) {
        data = { ...data, description };
      }
      if (weather) {
        data = { ...data, weather };
      }
      if (reason) {
        data = { ...data, reason: [reason] };
      }

      this.props.reportSegment(data).then((success) => {
        if (success) {
          this.handleAfterSubmit();
        }
      });
    }
  };

  render() {
    const { trafficReport, openedPopover } = this.props;
    const {
      error,
      velocity,
      description,
      addressData,
      weather,
      reason,
      markerData,
      isDrawable,
      multiReportData,
    } = trafficReport.report;
    const disableSubmit = trafficReport.isFetching ||
                          velocity === '' ||
                          (!isDrawable && markerData.length < 2) ||
                          (isDrawable && multiReportData.length === 0) ||
                          Object.keys(error).length > 0

    return (
      <Container fluid>
        <Row>
          <Col xs='12' className='text-center'>
            <div className='row justify-content-between'>
              <h4 className='teal-text default-font'>
                Báo Cáo Tình Trạng Giao Thông
              </h4>
              <Link
                to='/'
                style={{
                  padding: '8px',
                  cursor: 'pointer',
                }}
              >
                <i className='fa fa-times' aria-hidden='true'></i>
              </Link>
            </div>
          </Col>
          <Col lg='7' xs='12'>
            <p className='font-weight-bold'>Chọn vị trí cảnh báo</p>
            <div className='trafficformmap' style={{ position: 'relative' }}>
              <ReportMap
                // handleLocationData={this.handleLocationData}
                // handleAddressData={this.handleAddressData}
                onError={(key, message) => {
                  if (message) {
                    error[key] = message;
                  } else delete error[key];
                  // this.setState({ error: { ...error } });
                  this.setReportState({ 'report.error': { ...error } });
                }}
                customRef={(ref) => (this.reportMap = ref)}
                mapContainerRef={this.mapContainerRef}
              />
              {(openedPopover || openedPopover === 0) && (
                <SegmentPopover
                  anchorEl={this.mapContainerRef.current}
                  mapRef={
                    this.reportMap &&
                    this.reportMap.mapRef.current.leafletElement
                  }
                />
              )}
            </div>
          </Col>
          <Col lg='5' xs='12'>
            {/* <h5>
              Click chọn những đoạn xảy ra ùn tắc giao thông trên bản đồ
            </h5> */}
            <Spin spinning={trafficReport.isFetching}>
              <Form onSubmit={this.onSubmit}>
                <Row>
                  <Col xs='12'>
                    <FormControl
                      style={{ width: '100%' }}
                      error={!!error.start}
                    >
                      {/* <InputLabel htmlFor="age-simple">
                              Vận tốc (km/h)
                            </InputLabel> */}
                      <TextField
                        error={!!error.start}
                        id='start-point'
                        label='Điểm bắt đầu'
                        // defaultValue="Chọn điểm bắt đầu trên bản đồ"
                        value={addressData[0]}
                        margin='normal'
                        disabled
                      />
                      <FormHelperText id='component-error-text'>
                        {error.start}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs='12'>
                    <FormControl
                      style={{ width: '100%' }}
                      error={!!error.end}
                    >
                      <TextField
                        error={!!error.end}
                        label='Điểm kết thúc'
                        value={addressData[1]}
                        margin='normal'
                        // type="number"
                        disabled={true}
                        id='end-point'
                        // defaultValue="Chọn điểm kết thúc trên bản đồ"
                      />
                      <FormHelperText id='component-error-text'>
                        {error.end}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs='12'>
                    <FormControl
                      style={{ width: '100%' }}
                      error={!!error.velocity}
                    >
                      <TextField
                        error={!!error.velocity}
                        id='velocity'
                        label='Vận tốc'
                        value={velocity}
                        onChange={this.onInput('velocity')}
                        margin='normal'
                        type='number'
                        InputProps={{
                          endAdornment: (
                            <InputAdornment position='end'>
                              km/h
                            </InputAdornment>
                          ),
                        }}
                      />
                      <FormHelperText id='component-error-text'>
                        {error.velocity}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs='12' className='mt-2'>
                    <FormControl style={{ width: '100%' }}>
                      <InputLabel htmlFor='age-simple'>
                        Thời tiết
                      </InputLabel>
                      <Select
                        error={!!error.weather}
                        value={weather}
                        onChange={this.onInput('weather')}
                        placeholder='Chọn tình trạng thời tiết'
                      >
                        {CascaderWeatherOptions.map((item) => (
                          <MenuItem key={item.value} value={item.value}>
                            {item.label}
                          </MenuItem>
                        ))}
                      </Select>
                      <FormHelperText id='component-error-text'>
                        {error.weather}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs='12' className='mt-2'>
                    <FormControl style={{ width: '100%' }}>
                      <InputLabel htmlFor='age-simple'>
                        Nguyên nhân
                      </InputLabel>
                      <Select
                        error={!!error.reason}
                        value={reason}
                        onChange={this.onInput('reason')}
                        placeholder='Chọn nguyên nhân'
                        disabled={parseInt(velocity) > 40 ? true : false}
                      >
                        {CascaderReasonOptions.map((item) => (
                          <MenuItem key={item.value} value={item.value}>
                            {item.label}
                          </MenuItem>
                        ))}
                      </Select>
                      <FormHelperText id='component-error-text'>
                        {error.reason}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs='12' className='mt-2'>
                    <FormControl style={{ width: '100%' }}>
                      <TextField
                        error={!!error.description}
                        id='description'
                        label='Mô tả (nếu có)'
                        value={description}
                        onChange={this.onInput('description')}
                        margin='normal'
                        disabled={parseInt(velocity) > 40 ? true : false}
                      />
                      <FormHelperText id='component-error-text'>
                        {error.description}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs='12' className='text-center'>
                    <FormControl style={{ width: '100%' }} error>
                      <FormHelperText
                        id='component-error-text'
                        className='text-center'
                        style={{ fontSize: '1.2em' }}
                      >
                        <span style={{ display: 'block' }}>
                          {error.distance}
                        </span>
                      </FormHelperText>
                    </FormControl>
                    <Button
                      type='submit'
                      disabled={disableSubmit}
                      style={{
                        borderRadius: '24px',
                        minWidth: '150px',
                        outline: 'none',
                      }}
                      variant='contained'
                      color='primary'
                      onClick={this.onSubmit}
                    >
                      Gửi
                    </Button>
                  </Col>
                </Row>
              </Form>
            </Spin>
          </Col>
        </Row>
        <div className='mt-3'>
          <Link to='/' className='text-primary font-weight-bold d-flex ml-2'>
            <AiFillHome />
            <p className='ml-2'>Trang chủ</p>
          </Link>
        </div>
      </Container>
    );
  }
}

const mapStateToProps = (state) => ({
  isLogin: state.app.isLogin,
  trafficReport: state.trafficStatus,
  openedPopover: state.trafficStatus.reportMap.openedPopover,
});

const mapDispatchToProps = {
  reportSegment: reportAction.postReportV2,
  setReportState: reportAction.changeField,
  cleanReportState: reportAction.cleanState,
  reportMultiSegment: reportAction.postMultiReport,
};

export default connect(mapStateToProps, mapDispatchToProps)(TrafficReportForm);
