import React from "react";
import { Container, Row, Col } from "reactstrap";
import { Form } from "formik";
import { Link } from "react-router-dom";
import ReportMap from "../map/ReportMap";
import { client, customEvents } from "../../functions/eventEmitter";
import { connect } from "react-redux";
import reportAction from '../../actions/traffic-status';
import { Spin } from "antd";
import { AiFillHome } from "react-icons/ai";
import _ from "lodash";
import {
  FormControl,
  TextField,
  FormHelperText,
  Button,
} from "@material-ui/core";
import SegmentPopover from "../shared/SegmentPopover";
import AudioReactRecorder, { RecordState } from 'audio-react-recorder';
import { ObjectID } from 'bson';
import axios from "axios";

const REACT_APP_DOLBY_API_KEY = process.env.REACT_APP_DOLBY_API_KEY;
class SpeechReportForm extends React.Component {
  constructor(props, context) {
    super(props, context);
    this.reportMap = null;
    this.mapContainerRef = React.createRef();
    this.state = {
      record: RecordState.STOP,
      link: '',
      complete: false,
      recordedBlob: null,
    };

    this.handleAfterSubmit = this.handleAfterSubmit.bind(this);
    this.onSubmit = this.onSubmit.bind(this);
    this.onStop = this.onStop.bind(this);
    this.onRecord = this.onRecord.bind(this);
    this.onRecorderClicked = this.onRecorderClicked.bind(this);
  }

  handleAfterSubmit = () => {
    client.emit(customEvents.reportSuccess);
    this.setState({
      record: RecordState.STOP,
      link: '',
      complete: false,
      recordedBlob: null
    });
    this.props.cleanReportState("report");
    if (this.reportMap) {
      this.reportMap.onRemoveLayer();
    }
  };

  handleEnhanceAudio = async (audioFile, objectId) => {
    const input = `dlb://${objectId}`
    const output = `dlb://enhanced${objectId}`
    const dolbyInputUrl = 'https://api.dolby.com/media/input';
    const dolbyEnhanceUrl = 'https://api.dolby.com/media/enhance';
    const serverUrl = await axios({
      method: 'POST',
      url: dolbyInputUrl,
      data: { url: input },
      headers: { 'x-api-key': REACT_APP_DOLBY_API_KEY }
    })
      .then(res => { return res.data.url })
      .catch(err => console.error(err))

    await axios({
      method: 'PUT',
      url: serverUrl,
      data: audioFile
    })
      .catch(err => console.error(err))

    await axios({
      method: 'POST',
      url: dolbyEnhanceUrl,
      data: { input: input, output: output },
      headers: { 'x-api-key': REACT_APP_DOLBY_API_KEY }
    })
  }

  onSubmit = (e) => {
    e.preventDefault();
    const {
      error,
      multiReportData,
    } = this.props.trafficReport.report;

    let errors = { ...error };
    if (!_.isEmpty(errors) || !this.state.complete) {
      return;
    }

    const { postSpeechReport } = this.props;
    const streetList = multiReportData.map(item => item.segments);
    let id = new ObjectID();
    const data = {
      segments: streetList,
      record: this.state.recordedBlob,
      speech_record_id: id
    }

    let audioFile = new File([this.state.recordedBlob], "audio.wav");
    this.handleEnhanceAudio(audioFile, id);

    // Call action to dispatch API
    postSpeechReport(data);
    this.handleAfterSubmit();
  };
  onStop = (recordedBlob) => {
    this.setState({
      link: recordedBlob.url,
      recordedBlob: recordedBlob.blob,
    });
  };
  onRecord = () => {
    if (this.state.record === RecordState.STOP) {
      this.setState({ record: RecordState.START });
    } else {
      this.setState({ record: RecordState.STOP, complete: true });
    }
  }
  onRecorderClicked = () => {
    const recorder = document.getElementById("recorder");
    if (recorder.classList.contains("recording")) {
      recorder.classList.remove("recording");
      recorder.classList.add("download");
      this.onRecord();
      setTimeout(() => {
        recorder.classList.remove("download");
        recorder.classList.add("out");
      }, 1000);
    }
    else if (
      !recorder.classList.contains("recording") &&
      !recorder.classList.contains("download")
    ) {
      recorder.classList.remove("out");
      recorder.classList.add("recording");
      this.onRecord();
    }
  }

  render() {
    const { trafficReport, openedPopover } = this.props;
    const {
      error,
      addressData,
    } = trafficReport.report;
    const disableSubmit = trafficReport.isFetching || !this.state.complete

    return (
      <Container fluid>
        <Row>
          <Col xs='12' className='text-center'>
            <div className='row justify-content-between'>
              <h4 className='teal-text default-font'>
                Báo Cáo Tình Trạng Giao Thông
              </h4>
              <Link
                to='/'
                style={{
                  padding: '8px',
                  cursor: 'pointer',
                }}
              >
                <i className='fa fa-times' aria-hidden='true'></i>
              </Link>
            </div>
          </Col>
          <Col xs="12" lg="7">
            <p className="font-weight-bold">Chọn vị trí cảnh báo</p>
            <div
              className="trafficformmap"
              style={{ position: "relative" }}
            >
              <ReportMap
                onError={(key, message) => {
                  if (message) {
                    error[key] = message;
                  } else delete error[key];
                }}
                customRef={(ref) => (this.reportMap = ref)}
                mapContainerRef={this.mapContainerRef}
              />
              {(openedPopover || openedPopover === 0) && (
                <SegmentPopover
                  anchorEl={this.mapContainerRef.current}
                  mapRef={
                    this.reportMap &&
                    this.reportMap.mapRef.current.leafletElement
                  }
                />
              )}
            </div>
          </Col>
          <Col xs="12" lg="5">
            <Spin spinning={trafficReport.isFetching}>
              <Form onSubmit={this.onSubmit}>
                <Row>
                  <Col xs="12">
                    <FormControl
                      style={{ width: "100%" }}
                      error={!!error.start}
                    >
                      <TextField
                        error={!!error.start}
                        id="start-point"
                        label="Điểm bắt đầu"
                        value={addressData[0]}
                        margin="normal"
                        disabled
                      />
                      <FormHelperText id="component-error-text">
                        {error.start}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs="12">
                    <FormControl
                      style={{ width: "100%" }}
                      error={!!error.end}
                    >
                      <TextField
                        error={!!error.end}
                        label="Điểm kết thúc"
                        value={addressData[1]}
                        margin="normal"
                        // type="number"
                        disabled={true}
                        id="end-point"
                      // defaultValue="Chọn điểm kết thúc trên bản đồ"
                      />
                      <FormHelperText id="component-error-text">
                        {error.end}
                      </FormHelperText>
                    </FormControl>
                  </Col>
                  <Col xs="12">
                    <Row className="justify-content-center mt-5" noGutters>
                      <div id="recorder" onClick={this.onRecorderClicked}>
                        <img
                          id="record"
                          src="https://bassets.github.io/record.svg"
                          alt="voice report"
                        />
                        <img
                          id="arrow"
                          src="https://bassets.github.io/arrow.svg"
                          alt="replay"
                        />
                      </div>
                    </Row>
                  </Col>
                  <Col xs="12" className="automargin nopadding">
                    <Row id="menu" className="justify-content-center">
                      <AudioReactRecorder
                        state={this.state.record}
                        onStop={this.onStop}
                        canvasWidth="0px"
                        canvasHeight="0px"
                      />
                      {this.state.complete ? (
                        <div style={{ position: "relative", marginTop: "2vh", marginBottom: "2vh" }}>
                          <audio controls src={this.state.link}></audio>
                        </div>
                      ) : (
                        <div></div>
                      )}
                    </Row>
                  </Col>
                  <Col xs="12">
                    <Row className="justify-content-center mt-5" noGutters>
                      <FormControl style={{ width: "100%" }} error>
                        <FormHelperText
                          id="component-error-text"
                          className="text-center"
                          style={{ fontSize: "1.2em" }}
                        >
                          <span style={{ display: "block" }}>
                            {error.distance}
                          </span>
                        </FormHelperText>
                      </FormControl>
                      <Button
                        type="submit"
                        disabled={disableSubmit}
                        style={{
                          borderRadius: "24px",
                          minWidth: "150px",
                          outline: "none",
                        }}
                        variant="contained"
                        color="primary"
                        onClick={this.onSubmit}
                      >
                        Gửi
                      </Button>
                    </Row>
                  </Col>
                </Row>
              </Form>
            </Spin>
          </Col>
        </Row>
        <div className="mt-3">
          <Link to="/" className="text-primary font-weight-bold d-flex ml-2">
            <AiFillHome />
            <p className="ml-2">Trang chủ</p>
          </Link>
        </div>
      </Container>
    );
  }
}

const mapStateToProps = (state) => ({
  trafficReport: state.trafficStatus,
  openedPopover: state.trafficStatus.reportMap.openedPopover,
});

const mapDispatchToProps = {
  postSpeechReport: reportAction.postSpeechReport,
  cleanReportState: reportAction.cleanState,
};

export default connect(mapStateToProps, mapDispatchToProps)(SpeechReportForm);
