import React, { createRef } from 'react';
import { Map, TileLayer, Marker, Tooltip } from 'react-leaflet';
import 'leaflet/dist/leaflet.css';
import L from 'leaflet';
import locationIcon from '../../static/images/location.png';
import locationIcon2 from '../../static/images/location2.png';
import { LocateControl } from './fragments/LocateControl';
import locateIconImg from '../../static/images/circle.png';
import SearchBar from './fragments/SearchBar';
import { ZoomControlCustom } from './fragments/ZoomControlCustom';
import { connect } from 'react-redux';
import { segmentActions } from '../../actions/segment';
import redirectActions from '../../actions/redirect';
import { utils } from '../../modules/utils';
import { message } from 'antd';
import './style.css';
import trafficStatus from '../../actions/traffic-status';
import { drawPolyline, clearMap } from '../../modules/mapFunctions';
import { FaPencilAlt, FaHandPointer } from 'react-icons/fa';
import { GrPowerReset } from 'react-icons/gr';
import errorHandle from '../../components/shared/errorHandle';
import constants from '../../constants';

delete L.Icon.Default.prototype._getIconUrl;

L.Icon.Default.mergeOptions({
  iconRetinaUrl: require('leaflet/dist/images/marker-icon-2x.png'),
  iconUrl: require('leaflet/dist/images/marker-icon.png'),
  shadowUrl: require('leaflet/dist/images/marker-shadow.png'),
});

const icon = {
  1: new L.Icon({
    iconUrl: locationIcon2,
    iconSize: new L.Point(25, 41),
    className: 'icon-custom',
  }),
  0: new L.Icon({
    iconUrl: locationIcon,
    iconSize: new L.Point(25, 41),
    className: 'icon-custom',
  }),
};

const locateIcon = new L.Icon({
  iconUrl: locateIconImg,
  iconSize: new L.Point(16, 16),
  iconAnchor: [10, 10],
});

class ReportMap extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      zoom: 16,
      mapURL: 'https://mt3.google.com/vt/lyrs=m&x={x}&y={y}&z={z}',
      size: {
        height: window.innerWidth > 500 ? '400px' : '240px',
      },
      draggable: true,
      markerData: [],
      addressData: [],
      center: [10.7725104, 106.6575988],
      locateCoords: [10.7725104, 106.6575988],
      coordsList: [],
      coordsToDraw: {},
      drawMarkerPosition: [],
      mapDragging: true,
      isDrawing: false,
      drawable: false,
      drawListIndex: 0,
    };
    if (props.customRef) {
      props.customRef(this);
    }
  }

  mapRef = createRef();
  map = null;

  setZoomLevel = () => {
    const zoom = this.mapRef.current.leafletElement.getZoom();
    this.setState({
      zoom: zoom,
    });
  };

  componentDidMount() {
    navigator.geolocation.getCurrentPosition((location) => {
      const { coords } = location;
      const { latitude, longitude } = coords;
      this.setState({
        center: [latitude, longitude],
        locateCoords: [latitude, longitude],
      });
      const event = { latlng: { lat: latitude, lng: longitude } };
      this.addMarker(event);
    });
    this.map = this.mapRef.current.leafletElement;
  }
  componentDidUpdate(prevProps) {
    const { openedPopover } = this.props;
    // fix bug mousedown event when display popover
    if (
      prevProps.openedPopover !== openedPopover &&
      (openedPopover || openedPopover === 0)
    ) {
      this.setState({
        isDrawing: false,
      });
    }
  }
  componentWillUnmount() {
    this.onRemoveLayer();
  }

  handleDirect = (markerData) => {
    const { directReport, setReportState } = this.props;
    if (markerData.length === 2) {
      const query = {
        slat: markerData[0].lat,
        slng: markerData[0].lng,
        elat: markerData[1].lat,
        elng: markerData[1].lng,
        type: 'distance',
      };
      const clearMapFunc = () => clearMap(this.map);
      directReport(
        this.map,
        query,
        this.state.drawable,
        clearMapFunc,
        setReportState
      );
    }
  };

  distanceHandler = (start, end) => {
    const distance = utils.getDistanceBetweenTwoCoords(start, end);
    const error = this.props.report.error;
    if (distance > 1000) {
      error.distance = 'Khoảng cách cảnh báo theo đường thẳng lớn hơn 5km';
      this.props.setReportState({ 'report.error': { ...error } });
    } else {
      delete error.distance;
      this.props.setReportState({ 'report.error': { ...error } });
    }
  };

  addMarker = (event) => {
    if (this.state.drawable) {
      return;
    }
    const coords = event.latlng;
    let { markerData, addressData, error } = this.props.report;
    if (markerData.length < 2) {
      this.props
        .findNearSegments({ lat: coords.lat, lng: coords.lng })
        .then((data) => {
          if (data) {
            let listSegments = [];
            data.forEach((item) => {
              listSegments = [...listSegments, ...item.polyline.coordinates];
            });

            let nearestCoords = null;
            let nearestDistance = 1e8;
            for (let i = 0; i < listSegments.length; i++) {
              const item = listSegments[i];
              const distance = utils.getDistanceBetweenToPoint(
                [coords.lng, coords.lat],
                [item[0], item[1]]
              );
              if (distance < nearestDistance) {
                nearestCoords = item;
              }
            }

            if (nearestCoords) {
              // const newCoords = {
              //   lat: nearestCoords[1],
              //   lng: nearestCoords[0],
              // };
              const newCoords = {
                lat: coords.lat,
                lng: coords.lng,
              };
              markerData = [...markerData, newCoords];
              // if (markerData.length === 2) {
              //   this.distanceHandler(markerData[0], markerData[1]);
              // }
              if (markerData.length === 1) {
                delete error.start;
              }
              if (markerData.length === 2) {
                delete error.end;
                // const [start, end] = markerData;
                // drawDirect(markerData, this.map);
              }
              this.props.setReportState({
                'report.markerData': [...markerData],
                'report.error': { ...error },
              });

              const geocoder = L.Control.Geocoder.nominatim();
              geocoder.reverse(newCoords, this.state.zoom, (results) => {
                let address = results[0].name;
                addressData[markerData.length - 1] = address;
                this.props.setReportState({
                  'report.addressData': [...addressData],
                });
                this.handleDirect(markerData);
              });
            } else {
              // message.error({
              //   content:
              //     'Vị trí cảnh báo bạn chọn không nằm trên một con đường',
              //   duration: 1,
              // });
              errorHandle('Vị trí này không được hỗ trợ');
              const key = addressData.length < 1 ? 'start' : 'end';
              error[key] = 'Vị trí này không được hỗ trợ';
              this.props.setReportState({ 'report.error': { ...error } });
            }
          }
        });
    }
  };

  updateMarker = (event) => {
    if (this.state.drawable) {
      return;
    }
    let { markerData, addressData, error } = this.props.report;
    const markerIndex = event.target.options.marker_index;
    const coords = event.target.getLatLng();
    const { lat, lng } = coords;
    this.props.findNearSegments({ lat, lng }).then((data) => {
      let listSegments = [];
      data.forEach((item) => {
        listSegments = [...listSegments, ...item.polyline.coordinates];
      });

      let nearestCoords = null;
      let nearestDistance = 1e8;
      for (let i = 0; i < listSegments.length; i++) {
        const item = listSegments[i];
        const distance = utils.getDistanceBetweenToPoint(
          [lng, lat],
          [item[0], item[1]]
        );
        if (distance < nearestDistance) {
          nearestCoords = item;
        }
      }
      if (nearestCoords) {
        // const newCoords = {
        //   lat: nearestCoords[1],
        //   lng: nearestCoords[0],
        // };
        const newCoords = {
          lat: coords.lat,
          lng: coords.lng,
        };
        markerData = [...markerData];
        markerData[markerIndex] = newCoords;
        this.props.setReportState({ 'report.markerData': [...markerData] });

        // if (markerData.length === 2) {
        //   this.distanceHandler(markerData[0], markerData[1]);
        // }

        const geocoder = L.Control.Geocoder.nominatim();
        const addressIndex = event.target.options.marker_index;

        geocoder.reverse(newCoords, this.state.zoom, (results) => {
          let address = results[0].name;
          addressData = [...addressData];
          addressData[addressIndex] = address;

          const key = addressIndex < 1 ? 'start' : 'end';
          delete error[key];
          this.props.setReportState({
            'report.addressData': [...addressData],
            'report.error': { ...error },
          });
          this.handleDirect(markerData);
        });
      } else {
        const addressIndex = event.target.options.marker_index;
        message.error({
          content: 'Vị trí cảnh báo bạn chọn không nằm trên một con đường',
          duration: 1,
        });
        const key = addressIndex < 1 ? 'start' : 'end';
        error[key] = 'Vị trí cảnh báo không nằm trên một con đường';
        this.props.setReportState({ 'report.error': { ...error } });
      }
    });
  };

  onLocate = (location) => {
    const { latitude, longitude } = location.coords;
    this.setState({ center: [latitude, longitude] });
  };

  clearMap = () => {
    clearMap(this.map);
  };

  onRemoveLayer = () => {
    this.props.setReportState({
      'report.markerData': [],
      'report.addressData': [
        constants.addressDataStartDefault,
        constants.addressDataEndDefault,
      ],

      'report.multiReportData': [],
    });
    this.setState({
      coordsToDraw: {},
      drawMarkerPosition: [],
      drawListIndex: 0,
      isDrawing: false,
    });
    this.props.setReportState({
      'report.error': {},
    });
    clearMap(this.map);
  };

  resetState = () => {
    this.setState({
      coordsToDraw: {},
      drawMarkerPosition: [],
      mapDragging: true,
      isDrawing: false,
      drawable: false,
      drawListIndex: 0,
    });
  };

  onSearch = (result) => {
    const { lat, lng } = result.position;
    // const firstMarker = { lat, lng };
    // const geocoder = L.Control.Geocoder.nominatim();

    // geocoder.reverse(firstMarker, this.state.zoom, (results) => {
    //   let address = results[0].name;
    //   const addressData = ['', 'Chọn điểm cuối trên bản đồ'];
    //   addressData[0] = address;
    //   this.props.setReportState({
    //     'report.addressData': addressData,
    //     'report.markerData': [firstMarker],
    //     'report.error': {},
    //   });
    //   this.resetState();
    //   clearMap(this.map);
    // });
    // this.props.setReportState({
    //   'report.addressData': ,
    //   'report.markerData': [],
    //   'report.error': {},
    // });
    // clearMap(this.map);
    this.onRemoveLayer();
    this.setState({
      center: [lat, lng],
    });
  };

  onZoomControlCustomChange = (value) => {
    if (value === 'plus' && this.state.zoom < 19) {
      this.setState({ zoom: this.state.zoom + 1 });
    } else if (this.state.zoom > 5) {
      this.setState({ zoom: this.state.zoom - 1 });
    }
  };

  onDragToDraw = (event) => {
    if (!this.state.drawable) {
      return;
    }
    const { lat, lng } = event.latlng;
    const { coordsToDraw, drawListIndex } = this.state;
    if (!coordsToDraw[drawListIndex]) {
      coordsToDraw[drawListIndex] = [];
    }
    coordsToDraw[drawListIndex].push([lng, lat]);
    this.setState({ coordsToDraw: { ...coordsToDraw } });
    clearMap(this.map, `report-${drawListIndex}`);
    drawPolyline(coordsToDraw[drawListIndex], this.map, {
      onEachFeature: (feature, layer) => {
        layer.type = `line-report-${drawListIndex}`;
        layer.tag = 'GeoJSON';
        layer.on({
          click: (event) => {
            layer.remove();
          },
          mouseover: (event) => {
            event.target.setStyle({
              weight: 6,
              width: 30,
              color: 'blue',
            });
            // event.target.bringToBack();
          },
          mouseout: (event) => {
            event.target.setStyle({
              weight: 3,
              color: 'blue',
            });
          },
        });
      },
    });
  };

  onmousemove = (event) => {
    if (!this.state.drawable) {
      return;
    }
    const { lat, lng } = event.latlng;
    this.setState({ drawMarkerPosition: [lat, lng] });
    const { isDrawing } = this.state;
    if (isDrawing) {
      this.onDragToDraw(event);
    }
  };
  onmousedown = (event) => {
    if (!this.state.drawable) {
      return;
    }
    this.setState({ mapDragging: false, isDrawing: true });
  };
  onmouseup = (event) => {
    if (!this.state.drawable) {
      return;
    }
    this.setState({
      mapDragging: true,
      isDrawing: false,
    });
    const { directReport, setReportState } = this.props;
    const { coordsToDraw, drawListIndex } = this.state;

    if (
      coordsToDraw &&
      coordsToDraw[drawListIndex] &&
      coordsToDraw[drawListIndex].length > 0
    ) {
      const coordsStart = coordsToDraw[drawListIndex][0];
      const coordsEnd =
        coordsToDraw[drawListIndex][coordsToDraw[drawListIndex].length - 1];

      //direct
      const query = {
        slat: coordsStart[1],
        slng: coordsStart[0],
        elat: coordsEnd[1],
        elng: coordsEnd[0],
        type: 'distance',
      };
      const clearMapFunc = () => clearMap(this.map);
      directReport(
        this.map,
        query,
        this.state.drawable,
        clearMapFunc,
        setReportState
      );
      clearMap(this.map, `line-report-${drawListIndex}`);
      this.setState({
        drawListIndex: this.state.drawListIndex + 1,
      });
    }
  };
  onmouseout = () => {
    this.setState({ drawMarkerPosition: [] });
  };

  changeDrawable = () => {
    const drawable = this.state.drawable;
    // In pick mode change to draw mode
    if (!drawable) {
      this.setState({ draggable: false });
      this.props.setReportState({
        'report.markerData': [],
        'report.addressData': [
          constants.addressDataStartDefault,
          constants.addressDataEndDefault,
        ],
        'report.isDrawable': true,
      });
    } else {
      this.setState({
        draggable: true,
        coordsToDraw: {},
        drawMarkerPosition: [],
        drawListIndex: 0,
      });
      this.props.setReportState({
        'report.multiReportData': [],
        'report.isDrawable': false,
      });
    }
    this.props.setReportState({
      'report.error': {},
    });
    this.setState({ drawable: !drawable });
    clearMap(this.map);
  };

  render() {
    const { markerData } = this.props.report;
    const { mapDragging, drawable } = this.state;
    const { mapContainerRef } = this.props;
    return (
      <>
        <SearchBar
          onSearch={this.onSearch}
          recommendStyle={{
            position: 'absolute',
            zIndex: 100,
            width: 'calc(100% - 6px)',
          }}
        />
        <div className='map-report' ref={mapContainerRef}>
          <div className='map-bottom-right-container'>
            <div
              className='zoom-control-item'
              style={{ marginBottom: '3px' }}
              onClick={this.onRemoveLayer}
            >
              <GrPowerReset style={{ margin: 'auto' }} />
            </div>
            <div
              className='zoom-control-item'
              style={{ marginBottom: '3px' }}
              onClick={this.changeDrawable}
            >
              {!drawable ? (
                <FaPencilAlt style={{ margin: 'auto' }} />
              ) : (
                <FaHandPointer style={{ margin: 'auto' }} />
              )}
            </div>
            <LocateControl onLocate={this.onLocate} />
            <ZoomControlCustom onChange={this.onZoomControlCustomChange} />
          </div>
          <Map
            zoom={this.state.zoom}
            style={{
              ...this.state.size,
              cursor: drawable ? `default` : '',
            }}
            onZoomEnd={this.setZoomLevel}
            ref={this.mapRef}
            maxZoom={19}
            minZoom={5}
            // onMoveEnd={this.getNewBounds}
            zoomControl={false}
            scrollWheelZoom={false}
            onClick={this.addMarker}
            center={this.state.center}
            onViewportChanged={(event) => {
              const { center } = event;
              this.setState({ center });
            }}
            onmousemove={this.onmousemove}
            onmousedown={this.onmousedown}
            onmouseup={this.onmouseup}
            onmouseout={this.onmouseout}
            dragging={mapDragging}
            oncontextmenu={(e) => {
              console.log('Context Menu already lock');
            }}
          >
            <TileLayer
              attribution='&copy; <a href="https://osm.org/copyright">OpenStreetMap</a> contributors'
              url={this.state.mapURL}
              detectRetina={true}
            />

            <Marker
              position={this.state.locateCoords}
              draggable={false}
              icon={locateIcon}
            />
            {markerData.map((element, index) => (
              <Marker
                key={index}
                marker_index={index}
                position={element}
                draggable={this.state.draggable}
                onDragend={this.updateMarker}
                icon={icon[index]}
              >
                <Tooltip>{index === 0 ? 'Điểm đầu' : 'Điểm cuối'}</Tooltip>
              </Marker>
            ))}
          </Map>
        </div>
      </>
    );
  }
}
const mapStateToProps = (state) => ({
  report: state.trafficStatus.report,
  openedPopover: state.trafficStatus.reportMap.openedPopover,
});
const mapDispatchToProps = {
  findNearSegments: segmentActions.findNearSegments,
  setReportState: trafficStatus.changeField,
  cleanReoprtState: trafficStatus.cleanState,
  directReport: redirectActions.directReport,
};

export default connect(mapStateToProps, mapDispatchToProps)(ReportMap);
