import React, { useState, useEffect } from 'react';
import { makeStyles, withStyles } from '@material-ui/core/styles';
import {
  Button,
  Popover,
  Typography,
  TextField,
  InputAdornment,
} from '@material-ui/core';
import DeleteIcon from '@material-ui/icons/Delete';
import CloseIcon from '@material-ui/icons/Close';
import SendIcon from '@material-ui/icons/Send';
import DoneIcon from '@material-ui/icons/Done';
import IconButton from '@material-ui/core/IconButton';
import { useSelector, useDispatch } from 'react-redux';
import trafficStatus from '../../actions/traffic-status';
import { clearDirectReportMap } from '../../modules/mapFunctions';
import { client, customEvents } from '../../functions/eventEmitter';
import constants from '../../constants';
import { teal } from '@material-ui/core/colors';

const useStyles = makeStyles((theme) => ({
  typography: {
    padding: theme.spacing(1),
  },
  inputContainer: {
    display: 'flex',
    justifyContent: 'space-between',
    padding: theme.spacing(1),
    width: '100%',
    '& button': {
      outline: 'none',
    },
  },
  textField: {
    width: 150,
  },
  doneIcon: {
    color: 'green',
  },

  buttonContainer: {
    display: 'flex',
    justifyContent: 'space-between',
    padding: theme.spacing(1),
    '& > button': {
      outline: 'none',
    },
  },
}));

const ColorButton = withStyles((theme) => ({
  root: {
    color: theme.palette.getContrastText(teal[500]),
    backgroundColor: teal[500],
    '&:hover': {
      backgroundColor: teal[700],
    },
  },
}))(Button);

const SegmentPopover = (props) => {
  const classes = useStyles();
  const dispatch = useDispatch();
  const { anchorEl, mapRef } = props;
  const openedPopover = useSelector(
    (state) => state.trafficStatus.reportMap.openedPopover
  );
  const {
    multiReportData,
    markerData,
    velocity,
    error: reportError,
  } = useSelector((state) => state.trafficStatus.report);

  const [currentVelocity, setCurrentVelocity] = useState(undefined);

  useEffect(() => {
    const currentReportData = multiReportData.find(
      (report) => report.id === openedPopover
    );
    if (currentReportData && currentReportData.velocity) {
      setCurrentVelocity(currentReportData.velocity);
    } else {
      setCurrentVelocity(velocity);
    }
  }, [velocity, openedPopover, multiReportData]);

  const open = openedPopover || openedPopover === 0 ? true : false;
  const error =
    currentVelocity && (currentVelocity <= 0 || currentVelocity > 40)
      ? true
      : false;

  const handleVelocityChange = (e) => {
    setCurrentVelocity(e.target.value);
  };

  const handleClose = () => {
    dispatch(
      trafficStatus.changeField({
        'reportMap.openedPopover': undefined,
      })
    );
  };

  const handleDeleteSegments = () => {
    if (openedPopover) {
      const newMultiReportData = multiReportData.filter(
        (item) => item.id !== openedPopover
      );
      dispatch(
        trafficStatus.changeField({
          'report.multiReportData': newMultiReportData,
        })
      );
      if (mapRef) {
        clearDirectReportMap(mapRef, 'report-direct', openedPopover, true);
      }
    } else if (openedPopover === 0) {
      dispatch(
        trafficStatus.changeField({
          'report.markerData': [],
          'report.addressData': [
            constants.addressDataStartDefault,
            constants.addressDataEndDefault,
          ],
        })
      );
      if (mapRef) {
        clearDirectReportMap(mapRef, 'report-direct', openedPopover, false);
      }
    }
    handleClose();
  };

  const changeReportVelocity = () => {
    let isClearVelocityError = true;
    const reportErrors = { ...reportError };
    if (openedPopover) {
      const indexReportChanged = multiReportData.findIndex(
        (report) => report.id === openedPopover
      );
      if (indexReportChanged >= 0 || currentVelocity) {
        const newMultiReportData = [...multiReportData];
        newMultiReportData[indexReportChanged].velocity = currentVelocity;
        dispatch(
          trafficStatus.changeField({
            'report.multiReportData': newMultiReportData,
          })
        );

        for (const report of newMultiReportData) {
          if (!report.velocity) {
            isClearVelocityError = false;
            break;
          }
        }
      }
    } else if (openedPopover === 0) {
      dispatch(
        trafficStatus.changeField({
          'report.velocity': currentVelocity,
        })
      );
    }
    if(isClearVelocityError){
      delete reportErrors.velocity
    }
    dispatch(
      trafficStatus.changeField({
        'report.error': reportErrors,
      })
    );
    handleClose();
  };

  const handleFastReport = () => {
    if (openedPopover) {
      const indexReportChanged = multiReportData.findIndex(
        (report) => report.id === openedPopover
      );
      if (indexReportChanged >= 0 || currentVelocity > 0) {
        const postData = {
          segments: multiReportData[indexReportChanged].segments,
          velocity: currentVelocity,
        };
        dispatch(trafficStatus.postFastReport(postData))
          .then((success) => {
            if (success) {
              client.emit(customEvents.reportSuccess);
              handleDeleteSegments();
            }
          })
          .catch((err) => console.error(err));
      }
    } else if (openedPopover === 0) {
      const [start, end] = markerData;
      const data = {
        currentLat: start.lat.toString(),
        currentLng: start.lng.toString(),
        nextLat: end.lat.toString(),
        nextLng: end.lng.toString(),
        velocity: currentVelocity,
      };
      dispatch(trafficStatus.postReportV2(data))
        .then((success) => {
          if (success) {
            client.emit(customEvents.reportSuccess);
            handleDeleteSegments();
          }
        })
        .catch((err) => console.error(err));
    }
  };

  return (
    <Popover
      open={open}
      anchorEl={anchorEl}
      onClose={handleClose}
      anchorOrigin={{
        vertical: 'center',
        horizontal: 'center',
      }}
      transformOrigin={{
        vertical: 'center',
        horizontal: 'center',
      }}
      onContextMenu={(e) => {
        e.preventDefault();
      }}
    >
      <Typography className={classes.typography}>
        Thao tác với đoạn đường này
      </Typography>
      <div className={classes.inputContainer}>
        <TextField
          label='Vận tốc'
          id='velocity-input'
          className={classes.textField}
          InputProps={{
            endAdornment: <InputAdornment position='end'>km/h</InputAdornment>,
          }}
          variant='outlined'
          size='small'
          type='number'
          error={error}
          value={currentVelocity}
          onChange={handleVelocityChange}
        />
        <IconButton
          className={classes.doneIcon}
          onClick={changeReportVelocity}
          disabled={error || !currentVelocity}
        >
          <DoneIcon />
        </IconButton>
      </div>
      {error && (
        <Typography
          className={classes.typography}
          style={{ fontSize: 12, color: 'red', paddingTop: 0, marginTop: 0 }}
        >{`v > 0 hoặc v <= 40`}</Typography>
      )}
      <div className={classes.buttonContainer}>
        <ColorButton variant='contained' onClick={handleClose} color='primary'>
          <CloseIcon fontSize='small' />
        </ColorButton>
        <Button
          variant='contained'
          color='secondary'
          onClick={handleDeleteSegments}
        >
          <DeleteIcon fontSize='small' />
        </Button>
        <Button
          variant='contained'
          color='primary'
          disabled={error || !currentVelocity}
          onClick={handleFastReport}
        >
          <SendIcon fontSize='small' />
        </Button>
      </div>
    </Popover>
  );
};
export default SegmentPopover;
