import Config from "./config";
import AbortController from "abort-controller";

class Api {
  constructor(host) {
    this.host = host;
  }

  setToken(token) {
    this.token = token;
  }

  formatQuery(query) {
    let queryString = "?";
    for (let key in query) {
      if (typeof query[key] != "undefined") {
        queryString += `${key}=${query[key]}&`;
      }
    }
    return queryString;
  }

  fetchDataAllowAbort(api, method, body) {
    const abortController = new AbortController();
    const signal = abortController.signal;
    // abortController.abort();
    return {
      fetchData: () => this.fetchData(api, method, body, signal),
      abort: () => abortController.abort(),
    };
  }
  /**
   *
   * @param {*} data :
   * name:require,
   * email: optional when phone exist,
   * pasword: required,
   * phone: optional when email exist
   */

  async fetchData(api, method = "GET", body, signal) {
    if (!api.includes(this.host)) {
      api = this.host + api;
    }
    try {
      const options = {
        method,
        headers: {
          Accept: "application/json",
        },
        // signal,
        body,
      };
      // if (body) {
      //   options.body = body;
      // }
      if (signal) {
        options.signal = signal;
      }
      if (typeof body === "string") {
        options.headers["Content-Type"] = "application/json";
      } else if (body instanceof FormData) {
        // options.headers['Content-Type'] = 'multipart/form-data;boundary=----WebKitFormBoundaryyrV7KO0BoCBuDbTL';
      }
      if (this.token) {
        options.headers["Authorization"] = "Bearer " + this.token;
      }

      const response = await fetch(api, options);
      try {
        const json = await response.json();
        if (json.code === 200 || json.code === 201) {
          console.log("Response of", api, json);
          return { ...json, code: json.code, success: response.ok };
        } else {
          console.log("response error", response);
          throw json;
        }
      } catch (err) {
        console.log("response ", response);
        throw err;
      }
    } catch (err) {
      console.log("Response error'", err);
      return { code: -1, errors: err, success: false };
    }
  }

  register(data) {
    const api = this.host + "/api/auth/register";
    return this.fetchData(api, "POST", JSON.stringify(data));
  }

  /**
   *
   * @param {*} data
   * @param {'basic'|'google'|'facebook'} type
   */
  login(data, type = "basic") {
    let suffix = "";
    if (type === "basic") {
      suffix = "/api/auth/login";
    }
    if (type === "google") {
      suffix = "/api/auth/login-with-google";
    }
    if (type === "facebook") {
      suffix = "/api/auth/login-with-facebook";
    }
    const api = this.host + suffix;
    return this.fetchData(api, "POST", JSON.stringify(data));
  }

  /***********************USER*********************/

  getUserInfo() {
    const api = this.host + "/api/user/get-user-info";
    return this.fetchData(api);
  }

  /***********************upload file*********************/

  uploadFile(file) {
    const api = this.host + "/api/file/upload";
    let formData = new FormData();
    formData.append("file", file);
    return this.fetchData(api, "POST", formData);
  }

  /***********************traffic status*********************/
  postSpeechReport(data) {
    const api = this.host + "/api/report/speech-report";

    let formData = new FormData();
    formData.append('segments', JSON.stringify(data.segments));
    formData.append('record', data.record, 'record');
    formData.append('speech_record_id', data.speech_record_id);

    return this.fetchData(api, "POST", formData);
  }

  postReport(data) {
    const api = this.host + "/api/report/segment/here";
    return this.fetchData(api, "POST", JSON.stringify(data));
  }
  postReportV2(data) {
    const api = this.host + "/api/report/segment/user-report";
    return this.fetchData(api, "POST", JSON.stringify(data));
  }

  postMultiReport(data) {
    const api = this.host + "/api/report/segment/user-multi-report";
    return this.fetchData(api, "POST", JSON.stringify(data));
  }

  postFastReport(data) {
    const api = this.host + "/api/report/segment/user-fast-report";
    return this.fetchData(api, "POST", JSON.stringify(data));
  }

  getWeatherInfo() {
    let api = this.host + `/api/weather-info/get-info`;
    return this.fetchData(api, "GET");
  }

  getTrafficStatus(query, addSignal, isV2 = false) {
    if (!query.limit) query.limit = 10000;
    let queryString = this.formatQuery(query);
    let api = this.host + `/api/traffic-status/get-status${queryString}`;
    if (isV2) {
      api = this.host + `/api/traffic-status/get-status-v2${queryString}`;
    }
    return this.fetchDataAllowAbort(api, "GET");
  }

  redirect(query) {
    let queryString = this.formatQuery(query);
    // const api = this.host + `/api/segment/find-street${queryString}`;
    const api = this.host + `/api/segment/dynamic-routing${queryString}`;
    return this.fetchData(api);
  }
  /**
   *
   * @param {SegmentFindNear} query
   */
  findNearSegments(query) {
    if (!query.max_distance) {
      query.max_distance = 500;
    }
    const queryString = this.formatQuery(query);
    const api = this.host + `/api/segment/find-near${queryString}`;
    return this.fetchData(api);
  }

  getVOHNews() {
    const api = this.host + '/api/voh/get-news';
    return this.fetchData(api);
  }
}

const api = new Api(Config.BACKEND_HOST);
export default api;

/**
 * @typedef {object} SegmentFindNear
 * @property {string} lat
 * @property {string} lng
 * @property {number} max_distance //meter
 */

/**
 * @typedef {object} Coordinate
 * @property {string} lat
 * @property {string} lng
 */
